from typing import List

from blue_options.terminal import show_usage, xtra


args = [
    "[--overwrite 1]",
    "[--verbose 1]",
]


def options(mono: bool):
    return "".join(
        [
            xtra("count=<-1>,~download,dryrun,", mono=mono),
            "gif",
            xtra(",model=<model-id>,", mono=mono),
            "publish",
            xtra(",~upload", mono=mono),
        ]
    )


def help_detect(
    tokens: List[str],
    mono: bool,
) -> str:
    return show_usage(
        [
            "vanwatch",
            "detect",
            f"[{options(mono)}]",
            "[.|<object-name>]",
        ]
        + args,
        "detect objects in <object-name>.",
        mono=mono,
    )



from typing import List
import os
import requests
from tqdm import tqdm
from bs4 import BeautifulSoup

from blueness import module
from blue_objects import file
from blue_geo.file import load_geodataframe, save_geojson

from vancouver_watching import NAME
from vancouver_watching.QGIS import label_of_camera
from vancouver_watching.logger import logger

NAME = module.name(__file__, NAME)


def discover_cameras_vancouver_style(
    filename: str,
    prefix: str,
    count: int = -1,
) -> bool:
    logger.info(
        "{}.discover_cameras({}{}) -vancouver-style-> {}".format(
            NAME,
            prefix,
            "" if count == -1 else f"count={count}",
            filename,
        )
    )

    success, gdf = load_geodataframe(filename)
    if not success:
        return False

    list_of_cameras = []
    list_of_labels = []
    failed_locations = []
    row_count = 0
    for _, row in tqdm(gdf.iterrows()):
        list_of_cameras_ = []

        try:
            html_page = requests.get(row["url"])

            # https://towardsdatascience.com/a-tutorial-on-scraping-images-from-the-web-using-beautifulsoup-206a7633e948
            soup = BeautifulSoup(html_page.content, "html.parser")

            list_of_cameras_ = [
                f"{prefix}{src}"
                for src in [img["src"] for img in soup.find_all("img")]
                if src.startswith("cameraimages")
            ]
        except Exception as e:
            failed_locations += [row["url"]]
            logger.error(f"failed: {e}: {row['url']}")

        list_of_cameras += [",".join(list_of_cameras_)]
        list_of_labels += [
            label_of_camera(
                row["url"],
                row["name"],
                list_of_cameras_,
            )
        ]

        row_count += 1
        if count != -1 and row_count >= count:
            break

    if failed_locations:
        logger.error(f"{len(failed_locations)} location(s) failed.")
    logger.info(
        "found {} camera(s) in {} location(s).".format(
            len(
                [camera for camera in (",".join(list_of_cameras)).split(",") if camera]
            ),
            len(gdf),
        )
    )

    # for safety when count != -1
    gdf["cameras"] = (list_of_cameras + [""] * len(gdf))[: len(gdf)]
    gdf["label"] = (list_of_labels + [""] * len(gdf))[: len(gdf)]

    return save_geojson(filename, gdf)


def get_list_of_targets() -> List[str]:
    return [
        file.name(filename)
        for filename in file.list_of(
            os.path.join(
                file.path(__file__),
                "../.abcli/discovery/*.sh",
            )
        )
    ]



from typing import List

from blue_options.terminal import show_usage, xtra

from vancouver_watching.discover.functions import get_list_of_targets

target_details = {
    "target: {}".format(" | ".join(get_list_of_targets())): [],
}


def help_discover(
    tokens: List[str],
    mono: bool,
) -> str:
    options = "".join(
        [
            "target=<target>",
            xtra(",count=<-1>,dryrun,~tag,~upload", mono=mono),
        ]
    )

    return show_usage(
        [
            "vanwatch",
            "discover",
            f"[{options}]",
            "[-|<object-name>]",
            "[<args>]",
        ],
        "discover <target> -> <object-name>.",
        target_details,
        mono=mono,
    )



from typing import List

from blue_options.terminal import show_usage, xtra

from vancouver_watching.help.discover import target_details
from vancouver_watching.help.detect import (
    args as detect_args,
    options as detect_options,
)


def help_ingest(
    tokens: List[str],
    mono: bool,
) -> str:
    options = "".join(
        [
            "target=<target>",
            xtra(",count=<-1>,~download,dryrun,~upload", mono=mono),
        ]
    )

    return show_usage(
        [
            "vanwatch",
            "ingest",
            f"[{options}]",
            "[-|<object-name>]",
            f"[detect,{detect_options(mono=mono)}]",
        ]
        + detect_args,
        "ingest <target> -> <object-name>.",
        target_details,
        mono=mono,
    )



# to avoid import complications.
# from blue_objects.env import ABCLI_PUBLIC_PREFIX
ABCLI_PUBLIC_PREFIX = "https://kamangir-public.s3.ca-central-1.amazonaws.com"

NAME = "vancouver_watching"

ICON = "🌈"

DESCRIPTION = f"{ICON} Vancouver Watching with AI."

VERSION = "3.512.1"

REPO_NAME = "vancouver-watching"

MARQUEE = f"{ABCLI_PUBLIC_PREFIX}/test_vancouver_watching_ingest/animation.gif"

ALIAS = "vanwatch"


def fullname() -> str:
    return f"{NAME}-{VERSION}"



import os
import numpy as np
from datetime import datetime
from tqdm import tqdm
from typing import Tuple
import matplotlib.pyplot as plt
import pandas as pd
import glob

from blueness import module
from blue_options import fullname
from blue_objects import file, objects
from blue_objects.metadata import post_to_object

from vancouver_watching import NAME, VERSION
from vancouver_watching.logger import logger

NAME = module.name(__file__, NAME)


def label_of_camera(
    location_url,
    location_name,
    list_of_cameras,
):
    return '<a href="{}">{}</a><br/> {}'.format(
        location_url,
        location_name,
        f'<img src="{list_of_cameras[0]}">' if list_of_cameras else "camera not found.",
    )



import os
import os.path
from tqdm import tqdm

from blueness import module
from blue_objects import file

from vancouver_watching import NAME
from vancouver_watching.target.classes import Target
from vancouver_watching.logger import logger

NAME = module.name(__file__, NAME)


def ingest_target(
    target: Target,
    count: int = -1,
) -> bool:
    logger.info("{}.ingest({})".format(NAME, count))
    counter = 0
    for mapid in tqdm(target.metadata):
        for filename, metadata in target.metadata[mapid]["cameras"].items():
            url = metadata["url"]

            if target.do_dryrun:
                logger.info(url)
            elif not file.download(url, os.path.join(target.object_path, filename)):
                logger.error("bad url: {}.".format(url))

            counter += 1
            if counter >= count and count != -1:
                break
        if counter >= count and count != -1:
            break

    return True



import os

from blue_options import string
from blue_objects import file, README

from vancouver_watching import NAME, VERSION, ICON, REPO_NAME


items = README.Items(
    [
        {
            "name": "time-series",
            "marquee": f"https://kamangir-public.s3.ca-central-1.amazonaws.com/2024-01-06-20-39-46-73614/2024-01-06-20-39-46-73614-2X.gif?raw=true&random={string.random()}",
            "description": "example: [`2024-03-23-17-11-13-36842`](https://kamangir-public.s3.ca-central-1.amazonaws.com/2024-03-23-17-11-13-36842.tar.gz)",
            "url": "https://kamangir-public.s3.ca-central-1.amazonaws.com/vanwatch-cache-2024-02-28-21-04-19-26236.tar.gz",
        },
        {
            "name": "last build",
            "marquee": f"https://kamangir-public.s3.ca-central-1.amazonaws.com/test_vancouver_watching_ingest/animation.gif?raw=true&random={string.random()}",
            "url": f"https://kamangir-public.s3.ca-central-1.amazonaws.com/test_vancouver_watching_ingest/animation.gif?raw=true&random={string.random()}",
        },
    ]
)


def build():
    return README.build(
        items=items,
        path=os.path.join(file.path(__file__), ".."),
        ICON=ICON,
        NAME=NAME,
        VERSION=VERSION,
        REPO_NAME=REPO_NAME,
    )



import os
import os.path
from typing import List
import re
from collections import Counter
from tqdm import tqdm

from blueness import module
from blue_objects import file
from blue_objects.graphics.gif import generate_animated_gif

from vancouver_watching import NAME
from vancouver_watching.ai import Ultralytics_API
from vancouver_watching.target.classes import Target
from vancouver_watching.logger import logger


NAME = module.name(__file__, NAME)


def detect_in_target(
    target: Target,
    model_id: str,
    animated_gif: bool = False,
    count: int = -1,
    overwrite: bool = False,
) -> bool:
    logger.info(
        "{}.detect_in_target(model_id={},count={}){}{}".format(
            NAME,
            model_id,
            count,
            " +animated_gif" if animated_gif else "",
            " +overwrite" if overwrite else "",
        )
    )

    ultralytics_api = Ultralytics_API(
        model_id,
        target.do_dryrun,
        target.verbose,
    )

    list_of_images: List[str] = []
    counter: int = 0
    all_detections = {}
    for mapid in tqdm(target.metadata):
        detections = {}
        for filename, metadata in target.metadata[mapid]["cameras"].items():
            full_filename = os.path.join(target.object_path, filename)
            if not file.exists(full_filename):
                continue

            if target.do_dryrun:
                logger.info(f"dryrun: skipping {full_filename} ...")
                continue

            if overwrite or "inference" not in metadata:
                success, inference = ultralytics_api.infer(full_filename)
                if success:
                    metadata["inference"] = inference
                    list_of_images += [inference.get("render_filename", "")]

            if "inference" not in metadata:
                continue

            for thing, thing_count in Counter(
                [
                    thing["name"]
                    for thing in metadata["inference"]["images"][0]["results"]
                ]
            ).items():
                detections[thing] = detections.get(thing, 0) + thing_count

            if detections:
                logger.info(
                    "{}: {}".format(
                        mapid,
                        " + ".join(
                            [
                                f"{thing_count}*{thing}"
                                for thing, thing_count in detections.items()
                            ]
                        ),
                    )
                )

            for thing, thing_count in detections.items():
                all_detections[thing] = all_detections.get(thing, 0) + thing_count

                if thing not in target.gdf.columns:
                    target.gdf[thing] = 0
                    logger.info("+= {}".format(thing))

                target.gdf.loc[target.gdf["mapid"] == mapid, thing] += thing_count

            counter += 1
            if count != -1 and counter >= count:
                break
        if count != -1 and counter >= count:
            break

    logger.info(
        "all: {}".format(
            " + ".join(
                [
                    f"{thing_count}*{thing}"
                    for thing, thing_count in all_detections.items()
                ]
            )
        )
    )

    if not target.save_metadata():
        return False

    if not target.save_gdf():
        return False

    return not animated_gif or generate_animated_gif(
        [filename for filename in list_of_images if filename],
        os.path.join(target.object_path, f"{target.object_name}.gif"),
        frame_duration=500,
    )



import os
from blue_options.env import load_config, load_env

load_env(__name__)
load_config(__name__)

ULTRALYTICS_API_KEY = os.getenv(
    "ULTRALYTICS_API_KEY",
    "",
)

VANWATCH_DEFAULT_MODEL = os.getenv(
    "VANWATCH_DEFAULT_MODEL",
    "",
)

VANWATCH_TEST_OBJECT = os.getenv(
    "VANWATCH_TEST_OBJECT",
    "",
)

VANWATCH_QGIS_TEMPLATE = os.getenv(
    "VANWATCH_QGIS_TEMPLATE",
    "",
)
